//==============================================
//  copyright            : (C) 2003 by Will Stokes
//==============================================
//  This program is free software; you can redistribute it 
//  and/or modify it under the terms of the GNU General 
//  Public License as published by the Free Software 
//  Foundation; either version 2 of the License, or  
//  (at your option) any later version.         
//
//  As a special exception, Will Stokes gives permission to 
//  link this program with Qt non-commercial edition, and 
//  distribute the resulting executable, without including the 
//  source code for the Qt non-commercial edition in the 
//  source distribution. 
//==============================================

#ifndef BACKEND_PHOTO_H
#define BACKEND_PHOTO_H

//define image sizes
#define THUMBNAIL 1
#define PADDED_THUMBNAIL 2
#define SLIDESHOW 3
#define IMAGE 4

//--------------------
//forward declarations
class QString;
class QPixmap;
class QImage;
class QDomNode;
class Subalbum;
//--------------------

//=====================================
/*! \brief A photo consists of an image and a description.
 */
//=====================================

class Photo
{
//------------------------------------------------------
public:

  ///Sets default information
  Photo(Subalbum* subalbum, int number);

  ///Destructor
  ~Photo();
  //---------------------------- 
  ///Sets the image filename
  void setImageFilename(QString val);
    
  ///Sets the slideshow filename
  void setSlideshowFilename(QString val);
  //----------------------------
  ///Gets the image filename
  QString getImageFilename();
  
  ///Gets the slideshow filename
  QString getSlideshowFilename();
  //----------------------------
  ///Sets image, returns TRUE if successful, this method used when
  ///loading and image from file for the first time (aka when photo added to subalbum)
  bool setImage(QString imageName);
    
  ///Sets the image, uses loads thumbnail and slideshow 
  ///from file (no scaling), returns TRUE if successful. THis method used when
  ///loading from xml and no scaling should need to be done, we hope
  bool setImage(QString imageName,
                       QString slideshowName,
                       QString thumbnailName);
                      
  ///Sets the image. this method used when modifications have been preformed like
  ///cropping, etc. Recreates slideshow and thumnail forms, marks photo for saving
  bool setImage(QImage* fullImage);
  
  ///Sets the description
  void setDescription(QString val);
 
  ///Gets the description
  QString getDescription();
  
  ///Gets pointer to next photo
  Photo* getNext();
  
  ///Sets pointer to next photo
  void setNext(Photo* val);
  
  ///Gets image
  QImage* getImage(int size);
  
  ///Builds subalbum from XML DOM node
  void importFromDisk(QDomNode* root);
  
  ///Exports photo to xml
  void exportToXML(QTextStream& stream);
  
  ///Exports to thumbnail html format
  void exportThumbnailHTML(QTextStream& stream);
  
  ///Exports to slideshow html format
  void exportSlideshowHTML(QTextStream& stream);

  ///Rotates image clockwise by 90 degrees
  void rotate90();
  
  ///Rotates image clockwise by 270 degrees
  void rotate270();
  
  ///Flips image about horizontal axis
  void flipHorizontally();
  
  ///Flips image about vertical axis
  void flipVertically();

  ///Update thumbnail checksum
  void setThumbnailChecksum(QString val);
  
  ///Update slideshow checksum
  void setSlideshowChecksum(QString val);
  
  ///Update image checksum
  void setImageChecksum(QString val);

  ///Get thumbanil checksum
  QString getThumbnailChecksum();
  
  ///Get thumbanil checksum
  QString getSlideshowChecksum();
  
  ///Get image checksum
  QString getImageChecksum();
  
  ///Frees full image and slideshow versions to conserve memory
  void deallocateLargeImages();
  
  ///Sets if the image needs to be saved to its permanent location
  void setNeedsSavingVal(bool val);
  
  ///Returns if the image needs to be saved to its permament location
  bool getNeedsSavingVal();
  
  ///Returns initial photo number
  int getInitialPhotoNumber();
  
  ///Returns intiial subalbum number
  int getInitialSubalbumNumber();
  
  ///Sets initial photo number
  void setInitialPhotoNumber(int val);
  
  ///Sets initial subalbum number
  void setInitialSubalbumNumber(int val);
  
  ///Return actual slideshow image width (not including whitespace)
  int actualSlideshowWidth();
  
  ///Return actual slideshow image height (not including whitespace)
  int actualSlideshowHeight();  
  
  ///Sets actual slideshow image size
  void setActualSlideshowDimensions(int w, int h);
//------------------------------------------------------
private:
  ///Subalbum photo is in
  Subalbum* subalbum;

  //Initial photo number (when first created)
  int initialNumber;
  
  //Initial subalbum number of photo
  int initialSubalbumNumber;
      
  ///Pointer to next photo
  Photo* next;
  
  ///Longer description of subalbum
  QString description;
  
  ///Thumbnail used for saving to disk
  QImage* thumbnailImage;

  ///A padded thumbnail used for subalbum layout
  QImage* paddedThumbnailImage;
  
  ///Slideshow iamge
  QImage* slideshowImage;
  
  ///Actual image
  QImage* fullImage;
  
  ///MD5 checksums, ued to determine if image/thumbnail have been changed
  QString imageChecksum;
  QString slideshowChecksum;
  QString thumbnailChecksum;
  
  ///Image filename
  QString imageLocation;
  
  ///Slideshow filename
  QString slideshowLocation;
    
  ///Does the image need to be saved to the permanent location?
  bool needsSaving;
  
  ///Unpadded width of slideshow image
  int slideshowWidth;
  
  ///Unpadded height of slideshow image
  int slideshowHeight;
  
//------------------------------------------------------
};

#endif //BACKEND_PHOTO_H
